@@ -21,6 +21,7 @@ package org.apache.qpid.jms.integration;
 import static org.hamcrest.Matchers.equalTo;
 import static org.hamcrest.Matchers.notNullValue;
 import static org.junit.Assert.assertEquals;
+import static org.junit.Assert.assertFalse;
 import static org.junit.Assert.assertNotNull;
 import static org.junit.Assert.assertNull;
 import static org.junit.Assert.assertTrue;
@@ -30,7 +31,6 @@ import java.io.IOException;
 import java.nio.charset.StandardCharsets;
 import java.util.concurrent.CountDownLatch;
 import java.util.concurrent.TimeUnit;
-import java.util.concurrent.atomic.AtomicBoolean;
 import java.util.concurrent.atomic.AtomicReference;
 
 import javax.jms.Connection;
@@ -140,12 +140,20 @@ public class ConsumerIntegrationTest extends QpidJmsTestCase {
         final String BREAD_CRUMB = "ErrorMessage";
 
         try (TestAmqpPeer testPeer = new TestAmqpPeer();) {
-            final AtomicBoolean consumerClosed = new AtomicBoolean();
+            CountDownLatch consumerClosed = new CountDownLatch(1);
+            CountDownLatch exceptionFired = new CountDownLatch(1);
+
             JmsConnection connection = (JmsConnection) testFixture.establishConnecton(testPeer);
+            connection.setExceptionListener(new ExceptionListener() {
+                @Override
+                public void onException(JMSException exception) {
+                    exceptionFired.countDown();
+                }
+            });
             connection.addConnectionListener(new JmsDefaultConnectionListener() {
                 @Override
                 public void onConsumerClosed(MessageConsumer consumer, Throwable exception) {
-                    consumerClosed.set(true);
+                    consumerClosed.countDown();
                 }
             });
 
@@ -164,7 +172,7 @@ public class ConsumerIntegrationTest extends QpidJmsTestCase {
             testPeer.waitForAllHandlersToComplete(1000);
             assertTrue("consumer never closed.", Wait.waitFor(new Wait.Condition() {
                 @Override
-                public boolean isSatisified() throws Exception {
+                public boolean isSatisfied() throws Exception {
                     try {
                         consumer.getMessageListener();
                     } catch (IllegalStateException jmsise) {
@@ -180,7 +188,80 @@ public class ConsumerIntegrationTest extends QpidJmsTestCase {
                 }
             }, 10000, 10));
 
-            assertTrue("Consumer closed callback didn't trigger", consumerClosed.get());
+            assertTrue("Consumer closed callback didn't trigger", consumerClosed.await(2000, TimeUnit.MILLISECONDS));
+            assertFalse("JMS Exception listener shouldn't fire with no MessageListener", exceptionFired.await(20, TimeUnit.MILLISECONDS));
+
+            // Try closing it explicitly, should effectively no-op in client.
+            // The test peer will throw during close if it sends anything.
+            consumer.close();
+        }
+    }
+
+    @Test(timeout = 20000)
+    public void testRemotelyCloseConsumerWithMessageListenerFiresJMSExceptionListener() throws Exception {
+        final String BREAD_CRUMB = "ErrorMessage";
+
+        try (TestAmqpPeer testPeer = new TestAmqpPeer();) {
+            CountDownLatch consumerClosed = new CountDownLatch(1);
+            CountDownLatch exceptionFired = new CountDownLatch(1);
+
+            JmsConnection connection = (JmsConnection) testFixture.establishConnecton(testPeer);
+            connection.setExceptionListener(new ExceptionListener() {
+                @Override
+                public void onException(JMSException exception) {
+                    LOG.trace("JMS ExceptionListener: ", exception);
+                    exceptionFired.countDown();
+                }
+            });
+            connection.addConnectionListener(new JmsDefaultConnectionListener() {
+                @Override
+                public void onConsumerClosed(MessageConsumer consumer, Throwable exception) {
+                    consumerClosed.countDown();
+                }
+            });
+
+            // Use a zero prefetch to allow setMessageListener to not race link close
+            ((JmsDefaultPrefetchPolicy) connection.getPrefetchPolicy()).setAll(0);
+
+            testPeer.expectBegin();
+            Session session = connection.createSession(false, Session.AUTO_ACKNOWLEDGE);
+
+            // Create a consumer, then remotely end it afterwards.
+            testPeer.expectReceiverAttach();
+            testPeer.expectLinkFlow();
+            testPeer.remotelyDetachLastOpenedLinkOnLastOpenedSession(true, true, AmqpError.RESOURCE_DELETED, BREAD_CRUMB, 10);
+
+            Queue queue = session.createQueue("myQueue");
+            final MessageConsumer consumer = session.createConsumer(queue);
+            consumer.setMessageListener(new MessageListener() {
+
+                @Override
+                public void onMessage(Message message) {
+                }
+            });
+
+            // Verify the consumer gets marked closed
+            testPeer.waitForAllHandlersToComplete(1000);
+            assertTrue("consumer never closed.", Wait.waitFor(new Wait.Condition() {
+                @Override
+                public boolean isSatisfied() throws Exception {
+                    try {
+                        consumer.getMessageListener();
+                    } catch (IllegalStateException jmsise) {
+                        if (jmsise.getCause() != null) {
+                            String message = jmsise.getCause().getMessage();
+                            return message.contains(AmqpError.RESOURCE_DELETED.toString()) &&
+                                   message.contains(BREAD_CRUMB);
+                        } else {
+                            return false;
+                        }
+                    }
+                    return false;
+                }
+            }, 10000, 10));
+
+            assertTrue("Consumer closed callback didn't trigger",  consumerClosed.await(2000, TimeUnit.MILLISECONDS));
+            assertTrue("JMS Exception listener should have fired with a MessageListener", exceptionFired.await(2000, TimeUnit.MILLISECONDS));
 
             // Try closing it explicitly, should effectively no-op in client.
             // The test peer will throw during close if it sends anything.
@@ -841,7 +922,7 @@ public class ConsumerIntegrationTest extends QpidJmsTestCase {
             assertTrue("Consumer should close", Wait.waitFor(new Wait.Condition() {
 
                 @Override
-                public boolean isSatisified() throws Exception {
+                public boolean isSatisfied() throws Exception {
                     try {
                         consumer.getMessageSelector();
                         return false;
@@ -1005,7 +1086,7 @@ public class ConsumerIntegrationTest extends QpidJmsTestCase {
                 }
             });
 
-            boolean await = latch.await(3000, TimeUnit.MILLISECONDS);
+            boolean await = latch.await(3, TimeUnit.MINUTES);
             assertTrue("Messages not received within given timeout. Count remaining: " + latch.getCount(), await);
 
             assertNotNull("Expected IllegalStateException", asyncError.get());
@@ -1084,6 +1165,67 @@ public class ConsumerIntegrationTest extends QpidJmsTestCase {
         }
     }
 
+    @Test(timeout=20000)
+    public void testMessageListenerClosesItsConsumer() throws Exception {
+        final CountDownLatch latch = new CountDownLatch(1);
+        final CountDownLatch exceptionListenerFired = new CountDownLatch(1);
+        final AtomicReference<Throwable> error = new AtomicReference<>();
+
+        try (TestAmqpPeer testPeer = new TestAmqpPeer();) {
+            Connection connection = testFixture.establishConnecton(testPeer);
+            connection.start();
+
+            connection.setExceptionListener(new ExceptionListener() {
+                @Override
+                public void onException(JMSException exception) {
+                    LOG.trace("JMS ExceptionListener: ", exception);
+                    exceptionListenerFired.countDown();
+                }
+            });
+
+            testPeer.expectBegin();
+
+            final Session session = connection.createSession(false, Session.AUTO_ACKNOWLEDGE);
+            Queue destination = session.createQueue(getTestName());
+            connection.start();
+
+            testPeer.expectReceiverAttach();
+            testPeer.expectLinkFlowRespondWithTransfer(null, null, null, null, new AmqpValueDescribedType("content"), 1);
+
+            MessageConsumer consumer = session.createConsumer(destination);
+
+            testPeer.expectLinkFlow(true, true, equalTo(UnsignedInteger.valueOf(JmsDefaultPrefetchPolicy.DEFAULT_QUEUE_PREFETCH -1)));
+            testPeer.expectDisposition(true, new AcceptedMatcher());
+            testPeer.expectDetach(true, true, true);
+
+            consumer.setMessageListener(new MessageListener() {
+                @Override
+                public void onMessage(Message m) {
+                    try {
+                        consumer.close();
+                    } catch (Throwable t) {
+                        error.set(t);
+                        LOG.error("Unexpected error during close", t);
+                    }
+
+                    latch.countDown();
+                    LOG.debug("Async consumer got Message: {}", m);
+                }
+            });
+
+            assertTrue("Process not completed within given timeout", latch.await(3000, TimeUnit.MILLISECONDS));
+            assertNull("No error expected during close", error.get());
+
+            testPeer.waitForAllHandlersToComplete(2000);
+
+            testPeer.expectClose();
+            connection.close();
+
+            assertFalse("JMS Exception listener shouldn't have fired", exceptionListenerFired.await(20, TimeUnit.MILLISECONDS));
+            testPeer.waitForAllHandlersToComplete(2000);
+        }
+    }
+
     @Repeat(repetitions = 1)
     @Test(timeout=20000)
     public void testRecoverOrderingWithAsyncConsumer() throws Exception {
