@@ -20,6 +20,7 @@ package org.apache.qpid.jms.test.testpeer;
 
 import java.util.List;
 
+import org.apache.qpid.jms.test.testpeer.describedtypes.FrameDescriptorMapping;
 import org.apache.qpid.proton.amqp.Binary;
 import org.apache.qpid.proton.amqp.DescribedType;
 import org.apache.qpid.proton.amqp.Symbol;
@@ -41,21 +42,25 @@ public abstract class AbstractFrameFieldAndPayloadMatchingHandler extends Abstra
 
     private AmqpPeerRunnable _onCompletion;
 
+    private int _expectedFrameSize;
+
     protected AbstractFrameFieldAndPayloadMatchingHandler(FrameType frameType,
                                                 int channel,
+                                                int frameSize,
                                                 UnsignedLong numericDescriptor,
                                                 Symbol symbolicDescriptor)
     {
         super(numericDescriptor, symbolicDescriptor);
         _frameType = frameType;
         _expectedChannel = channel;
+        _expectedFrameSize = frameSize;
     }
 
     protected abstract void verifyPayload(Binary payload) throws AssertionError;
 
     @SuppressWarnings("unchecked")
     @Override
-    public final void frame(int type, int ch, DescribedType dt, Binary payload, TestAmqpPeer peer)
+    public final void frame(int type, int ch, int frameSize, DescribedType dt, Binary payload, TestAmqpPeer peer)
     {
         if(type == _frameType.ordinal()
            && (_expectedChannel == ANY_CHANNEL || _expectedChannel == ch)
@@ -84,6 +89,14 @@ public abstract class AbstractFrameFieldAndPayloadMatchingHandler extends Abstra
                 peer.assertionFailed(ae);
             }
 
+            if(_expectedFrameSize != 0 && _expectedFrameSize != frameSize) {
+                throw new IllegalArgumentException(String.format(
+                        "Frame size was not as expected. Expected: " +
+                        "type=%s, channel=%s, size=%s, descriptor=%s/%s but got: " +
+                        "type=%s, channel=%s, size=%s",
+                        _frameType.ordinal(), expectedChannelString(), _expectedFrameSize, getNumericDescriptor(), getSymbolicDescriptor(), type, ch, frameSize));
+            }
+
             if(_onCompletion != null)
             {
                 _onCompletion.run();
@@ -95,12 +108,15 @@ public abstract class AbstractFrameFieldAndPayloadMatchingHandler extends Abstra
         }
         else
         {
+            Object actualDescriptor = dt.getDescriptor();
+            Object mappedDescriptor = FrameDescriptorMapping.lookupMapping(actualDescriptor);
+
             throw new IllegalArgumentException(String.format(
                     "Frame was not as expected. Expected: " +
                     "type=%s, channel=%s, descriptor=%s/%s but got: " +
-                    "type=%s, channel=%s, descriptor=%s",
-                    _frameType.ordinal(), expectedChannelString(), getSymbolicDescriptor(), getNumericDescriptor(),
-                    type, ch, dt.getDescriptor()));
+                    "type=%s, channel=%s, descriptor=%s(%s)",
+                    _frameType.ordinal(), expectedChannelString(), getNumericDescriptor(), getSymbolicDescriptor(),
+                    type, ch, actualDescriptor, mappedDescriptor));
         }
     }
 
