@@ -1,4 +1,4 @@
-/**
+/*
  * Licensed to the Apache Software Foundation (ASF) under one or more
  * contributor license agreements.  See the NOTICE file distributed with
  * this work for additional information regarding copyright ownership.
@@ -31,14 +31,17 @@ import java.security.cert.X509Certificate;
 import org.apache.qpid.jms.transports.Transport;
 import org.apache.qpid.jms.transports.TransportListener;
 import org.apache.qpid.jms.transports.TransportOptions;
-import org.apache.qpid.jms.transports.TransportSslOptions;
+import org.apache.qpid.jms.util.QpidJMSTestRunner;
+import org.apache.qpid.jms.util.Repeat;
 import org.junit.Test;
+import org.junit.runner.RunWith;
 import org.slf4j.Logger;
 import org.slf4j.LoggerFactory;
 
 /**
- * Test basic functionality of the Netty based SSL Transport.
+ * Test basic functionality of the Netty based TCP Transport ruuing in secure mode (SSL).
  */
+@RunWith(QpidJMSTestRunner.class)
 public class NettySslTransportTest extends NettyTcpTransportTest {
 
     private static final Logger LOG = LoggerFactory.getLogger(NettySslTransportTest.class);
@@ -61,16 +64,19 @@ public class NettySslTransportTest extends NettyTcpTransportTest {
 
     @Override
     @Test(timeout = 60 * 1000)
-    public void testCreateWithNullOptionsUsesDefaults() throws Exception {
+    public void testCreateWithNullOptionsThrowsIAE() throws Exception {
         URI serverLocation = new URI("tcp://localhost:5762");
 
-        Transport transport = createTransport(serverLocation, testListener, null);
-        assertEquals(TransportSslOptions.INSTANCE, transport.getTransportOptions());
+        try {
+            createTransport(serverLocation, testListener, null);
+            fail("Should have thrown IllegalArgumentException");
+        } catch (IllegalArgumentException iae) {
+        }
     }
 
     @Test(timeout = 60 * 1000)
     public void testConnectToServerWithoutTrustStoreFails() throws Exception {
-        try (NettyEchoServer server = new NettyEchoServer(createServerOptions())) {
+        try (NettyEchoServer server = createEchoServer(createServerOptions())) {
             server.start();
 
             int port = server.getServerPort();
@@ -78,7 +84,7 @@ public class NettySslTransportTest extends NettyTcpTransportTest {
 
             Transport transport = createTransport(serverLocation, testListener, createClientOptionsWithoutTrustStore(false));
             try {
-                transport.connect();
+                transport.connect(null);
                 fail("Should not have connected to the server: " + serverLocation);
             } catch (Exception e) {
                 LOG.info("Connection failed to untrusted test server: {}", serverLocation);
@@ -95,21 +101,22 @@ public class NettySslTransportTest extends NettyTcpTransportTest {
     }
 
     @Test(timeout = 60 * 1000)
+    @Repeat(repetitions = 1)
     public void testConnectToServerUsingUntrustedKeyFails() throws Exception {
-        try (NettyEchoServer server = new NettyEchoServer(createServerOptions())) {
+        try (NettyEchoServer server = createEchoServer(createServerOptions())) {
             server.start();
 
             int port = server.getServerPort();
             URI serverLocation = new URI("tcp://localhost:" + port);
 
-            TransportSslOptions options = TransportSslOptions.INSTANCE.clone();
+            TransportOptions options = new TransportOptions();
 
             options.setTrustStoreLocation(OTHER_CA_TRUSTSTORE);
             options.setTrustStorePassword(PASSWORD);
 
             Transport transport = createTransport(serverLocation, testListener, options);
             try {
-                transport.connect();
+                transport.connect(null);
                 fail("Should not have connected to the server: " + serverLocation);
             } catch (Exception e) {
                 LOG.info("Connection failed to untrusted test server: {}", serverLocation);
@@ -123,7 +130,7 @@ public class NettySslTransportTest extends NettyTcpTransportTest {
 
     @Test(timeout = 60 * 1000)
     public void testConnectToServerClientTrustsAll() throws Exception {
-        try (NettyEchoServer server = new NettyEchoServer(createServerOptions())) {
+        try (NettyEchoServer server = createEchoServer(createServerOptions())) {
             server.start();
 
             int port = server.getServerPort();
@@ -131,13 +138,14 @@ public class NettySslTransportTest extends NettyTcpTransportTest {
 
             Transport transport = createTransport(serverLocation, testListener, createClientOptionsWithoutTrustStore(true));
             try {
-                transport.connect();
+                transport.connect(null);
                 LOG.info("Connection established to untrusted test server: {}", serverLocation);
             } catch (Exception e) {
-                fail("Should have connected to the server: " + serverLocation);
+                fail("Should have connected to the server at " + serverLocation + " but got exception: " + e);
             }
 
             assertTrue(transport.isConnected());
+            assertTrue(transport.isSecure());
 
             transport.close();
         }
@@ -148,25 +156,26 @@ public class NettySslTransportTest extends NettyTcpTransportTest {
 
     @Test(timeout = 60 * 1000)
     public void testConnectWithNeedClientAuth() throws Exception {
-        TransportSslOptions serverOptions = createServerOptions();
+        TransportOptions serverOptions = createServerOptions();
 
-        try (NettyEchoServer server = new NettyEchoServer(serverOptions, true)) {
+        try (NettyEchoServer server = createEchoServer(serverOptions, true)) {
             server.start();
 
             int port = server.getServerPort();
             URI serverLocation = new URI("tcp://localhost:" + port);
 
-            TransportSslOptions clientOptions = createClientOptions();
+            TransportOptions clientOptions = createClientOptions();
 
-            NettySslTransport transport = createTransport(serverLocation, testListener, clientOptions);
+            NettyTcpTransport transport = createTransport(serverLocation, testListener, clientOptions);
             try {
-                transport.connect();
+                transport.connect(null);
                 LOG.info("Connection established to test server: {}", serverLocation);
             } catch (Exception e) {
-                fail("Should have connected to the server: " + serverLocation);
+                fail("Should have connected to the server at " + serverLocation + " but got exception: " + e);
             }
 
             assertTrue(transport.isConnected());
+            assertTrue(transport.isSecure());
 
             // Verify there was a certificate sent to the server
             assertNotNull(server.getSslHandler().engine().getSession().getPeerCertificates());
@@ -185,27 +194,28 @@ public class NettySslTransportTest extends NettyTcpTransportTest {
     }
 
     private void doClientAuthAliasTestImpl(String alias, String expectedDN) throws Exception, URISyntaxException, IOException, InterruptedException {
-        TransportSslOptions serverOptions = createServerOptions();
+        TransportOptions serverOptions = createServerOptions();
 
-        try (NettyEchoServer server = new NettyEchoServer(serverOptions, true)) {
+        try (NettyEchoServer server = createEchoServer(serverOptions, true)) {
             server.start();
 
             int port = server.getServerPort();
             URI serverLocation = new URI("tcp://localhost:" + port);
 
-            TransportSslOptions clientOptions = createClientOptions();
+            TransportOptions clientOptions = createClientOptions();
             clientOptions.setKeyStoreLocation(CLIENT_MULTI_KEYSTORE);
             clientOptions.setKeyAlias(alias);
 
-            NettySslTransport transport = createTransport(serverLocation, testListener, clientOptions);
+            NettyTcpTransport transport = createTransport(serverLocation, testListener, clientOptions);
             try {
-                transport.connect();
+                transport.connect(null);
                 LOG.info("Connection established to test server: {}", serverLocation);
             } catch (Exception e) {
-                fail("Should have connected to the server: " + serverLocation);
+                fail("Should have connected to the server at " + serverLocation + " but got exception: " + e);
             }
 
             assertTrue(transport.isConnected());
+            assertTrue(transport.isSecure());
 
             Certificate[] peerCertificates = server.getSslHandler().engine().getSession().getPeerCertificates();
             assertNotNull(peerCertificates);
@@ -233,16 +243,16 @@ public class NettySslTransportTest extends NettyTcpTransportTest {
     }
 
     private void doConnectToServerVerifyHostTestImpl(boolean verifyHost) throws Exception, URISyntaxException, IOException, InterruptedException {
-        TransportSslOptions serverOptions = createServerOptions();
+        TransportOptions serverOptions = createServerOptions();
         serverOptions.setKeyStoreLocation(SERVER_WRONG_HOST_KEYSTORE);
 
-        try (NettyEchoServer server = new NettyEchoServer(serverOptions)) {
+        try (NettyEchoServer server = createEchoServer(serverOptions)) {
             server.start();
 
             int port = server.getServerPort();
             URI serverLocation = new URI("tcp://localhost:" + port);
 
-            TransportSslOptions clientOptions = createClientOptionsIsVerify(verifyHost);
+            TransportOptions clientOptions = createClientOptionsIsVerify(verifyHost);
 
             if (verifyHost) {
                 assertTrue("Expected verifyHost to be true", clientOptions.isVerifyHost());
@@ -252,7 +262,7 @@ public class NettySslTransportTest extends NettyTcpTransportTest {
 
             Transport transport = createTransport(serverLocation, testListener, clientOptions);
             try {
-                transport.connect();
+                transport.connect(null);
                 if (verifyHost) {
                     fail("Should not have connected to the server: " + serverLocation);
                 }
@@ -261,7 +271,7 @@ public class NettySslTransportTest extends NettyTcpTransportTest {
                     LOG.info("Connection failed to test server: {} as expected.", serverLocation);
                 } else {
                     LOG.error("Failed to connect to test server: " + serverLocation, e);
-                    fail("Should have connected to the server: " + serverLocation + ", but got: " + e);
+                    fail("Should have connected to the server at " + serverLocation + " but got exception: " + e);
                 }
             }
 
@@ -276,17 +286,27 @@ public class NettySslTransportTest extends NettyTcpTransportTest {
     }
 
     @Override
-    protected NettySslTransport createTransport(URI serverLocation, TransportListener listener, TransportOptions options) {
-        return new NettySslTransport(listener, serverLocation, options);
+    protected NettyTcpTransport createTransport(URI serverLocation, TransportListener listener, TransportOptions options) {
+        return new NettyTcpTransport(listener, serverLocation, options, true);
     }
 
     @Override
-    protected TransportSslOptions createClientOptions() {
+    protected NettyEchoServer createEchoServer(TransportOptions options) {
+        return createEchoServer(options, false);
+    }
+
+    @Override
+    protected NettyEchoServer createEchoServer(TransportOptions options, boolean needClientAuth) {
+        return new NettyEchoServer(options, true, needClientAuth);
+    }
+
+    @Override
+    protected TransportOptions createClientOptions() {
         return createClientOptionsIsVerify(false);
     }
 
-    protected TransportSslOptions createClientOptionsIsVerify(boolean verifyHost) {
-        TransportSslOptions options = TransportSslOptions.INSTANCE.clone();
+    protected TransportOptions createClientOptionsIsVerify(boolean verifyHost) {
+        TransportOptions options = new TransportOptions();
 
         options.setKeyStoreLocation(CLIENT_KEYSTORE);
         options.setKeyStorePassword(PASSWORD);
@@ -299,9 +319,10 @@ public class NettySslTransportTest extends NettyTcpTransportTest {
     }
 
     @Override
-    protected TransportSslOptions createServerOptions() {
-        TransportSslOptions options = TransportSslOptions.INSTANCE.clone();
+    protected TransportOptions createServerOptions() {
+        TransportOptions options = new TransportOptions();
 
+        // Run the server in JDK mode for now to validate cross compatibility
         options.setKeyStoreLocation(SERVER_KEYSTORE);
         options.setKeyStorePassword(PASSWORD);
         options.setTrustStoreLocation(SERVER_TRUSTSTORE);
@@ -312,8 +333,8 @@ public class NettySslTransportTest extends NettyTcpTransportTest {
         return options;
     }
 
-    protected TransportSslOptions createClientOptionsWithoutTrustStore(boolean trustAll) {
-        TransportSslOptions options = TransportSslOptions.INSTANCE.clone();
+    protected TransportOptions createClientOptionsWithoutTrustStore(boolean trustAll) {
+        TransportOptions options = new TransportOptions();
 
         options.setStoreType(KEYSTORE_TYPE);
         options.setTrustAll(trustAll);
