@@ -1,4 +1,4 @@
-/**
+/*
  * Licensed to the Apache Software Foundation (ASF) under one or more
  * contributor license agreements.  See the NOTICE file distributed with
  * this work for additional information regarding copyright ownership.
@@ -38,9 +38,6 @@ import org.junit.Test;
 import org.slf4j.Logger;
 import org.slf4j.LoggerFactory;
 
-/**
- *
- */
 public class JmsAutoAckTest extends AmqpTestSupport {
 
     protected static final Logger LOG = LoggerFactory.getLogger(JmsAutoAckTest.class);
@@ -65,7 +62,7 @@ public class JmsAutoAckTest extends AmqpTestSupport {
         assertTrue("Queued message not consumed.", Wait.waitFor(new Wait.Condition() {
 
             @Override
-            public boolean isSatisified() throws Exception {
+            public boolean isSatisfied() throws Exception {
                 return proxy.getQueueSize() == 0;
             }
         }));
@@ -97,7 +94,7 @@ public class JmsAutoAckTest extends AmqpTestSupport {
         assertTrue("Queued message not consumed.", Wait.waitFor(new Wait.Condition() {
 
             @Override
-            public boolean isSatisified() throws Exception {
+            public boolean isSatisfied() throws Exception {
                 return proxy.getQueueSize() == 0;
             }
         }));
@@ -113,7 +110,7 @@ public class JmsAutoAckTest extends AmqpTestSupport {
      * message and expect to see it again, ensure the third message is not seen
      * until after this.
      *
-     * @throws Exception
+     * @throws Exception on error during test.
      */
     @Test(timeout = 60000)
     public void testRecoverInOnMessage() throws Exception {
